"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC12F508 DIP8
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_singlePMarea import *

class Chip_Pic12F508dip8(microchip8_singlePMarea):

	logicalFlashSize = 0x400

	def __init__(self):
		microchip8_singlePMarea.__init__(self,
		chipPackage="DIP8",
		chipPinVCC=1,
		chipPinsVPP=4,
		chipPinGND=8,
		signature="",
		flashPageSize=0x200,
		flashPages=1,
		eepromPageSize=0,
		eepromPages=0,
		fuseBytes=2
		)
	
fuseDesc = (
	BitDescription(0, "FOSC[0] 00=LP, 01=XT, 10=INTOSC, 11=EXTRC"),
	BitDescription(1, "FOSC[1]"),
	BitDescription(2, "WDTE"),
	BitDescription(3, "!CP"),
	BitDescription(4, "MCLRE"),
	BitDescription(5, "Unused"),
	BitDescription(6, "Unused"),
	BitDescription(7, "Unused"),
	BitDescription(8, "Unused"),
	BitDescription(9, "Unused"),
	BitDescription(10, "Unused"),
	BitDescription(11, "Unused"),
)
ChipDescription(
	Chip_Pic12F508dip8,
	bitfile="microchip01dip8",
	chipID="pic12f508dip8",
	runtimeID=(0xDE02, 0x01),
	chipVendors="Microchip",
	description="PIC12F508",
	fuseDesc=fuseDesc, 	
	packages=(("DIP8", ""),),
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC16LF1902 dip28
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_splittedPMarea_hasResetPC import *

class Chip_Pic16LF1902dip28(microchip8_splittedPMarea_hasResetPC):
		
	nLatches = 8
    	def __init__(self):
	    	microchip8_splittedPMarea_hasResetPC.__init__(self,
			chipPackage="dip28",
			chipPinVCC=20,
			chipPinsVPP=1,
			chipPinGND=19,
			signature="\x22\x2c",
			flashPageSize=0x800,
			flashPages=1,
			eepromPageSize=0,
			eepromPages=0,
			fuseBytes=4
			)

fuseDesc = (
	BitDescription(0, "FOSC[0], 00=INTOSC, 11=ECH - High power mode"),
	BitDescription(1, "FOSC[1]"),
	BitDescription(2, "Unused"),
	BitDescription(3, "WDTE[0], 00=WDT disabled, 11=WDT enabled"),
	BitDescription(4, "WDTE[1]"),
	BitDescription(5, "nPWRTE"),
	BitDescription(6, "MCLRE, 1=nMCLR/Vpp pin is nMCLR, weak pull-up enabled, ignored if LVP=1 "),
	BitDescription(7, "nCP 1=program memory code protection is disabled"),
	BitDescription(8, "Unused"),
	BitDescription(9, "BOREN[0], 00=BOR disabled"),
	BitDescription(10, "BOREN[1]"),
	BitDescription(11, "nCLKOUTEN, 0=CLKOUT is enabled on CLKOUT pin"),
	BitDescription(12, "Unused"),
	BitDescription(13, "Unused"),
	BitDescription(14, "NA"),
	BitDescription(15, "NA"),
	
	BitDescription(16, "WRT[0], 11=Write protection off"),
	BitDescription(17, "WRT[1]"),
	BitDescription(18, "Unused"),
	BitDescription(19, "Unused"),
	BitDescription(20, "Unused"),
	BitDescription(21, "Unused"),
	BitDescription(22, "Unused"),
	BitDescription(23, "Unused"),
	BitDescription(24, "Unused"),
	BitDescription(25, "STVREN, 1=Stack overflow or underflow will cause a reset"),
	BitDescription(26, "BORV"),
	BitDescription(27, "nULPBOR, 1= Ultra low-power BOR is disabled"),
	BitDescription(28, "nDEBUG, 0=ICSPCLK and ICSPDAT are dedicated to the debugger"),
	BitDescription(29, "LVP 1=Low-voltage programming enabled"),
	BitDescription(30, "NA"),
	BitDescription(31, "NA"),
)

ChipDescription(
	Chip_Pic16LF1902dip28,
	bitfile="microchip01dip28",
	chipID="pic16lf1902dip28",
	runtimeID=(0xDE07, 0x01),
	chipVendors="Microchip",
	description="PIC16LF1902",
	packages=(("dip28", ""),),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



"""
#    TOP2049 Open Source programming suite
#
#    pic8_splittedPMarea - file for newer 8bit PIC MCUs
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chips.microchip8.microchip8_common import *

class microchip8_splittedPMarea(Chip_Microchip8_common):
	CMD_LOAD_CONFIGURATION = 0x00
	
	logicalFlashProgramMemorySize = 0x2000
	logicalFlashConfigurationMemorySize = 0x2000
	   
	CMD_BEGIN_INTERNALLY_TIMED_PROGRAMMING = 0x08
	CMD_BEGIN_EXTERNALLY_TIMED_PROGRAMMING = 0x18
	CMD_END_EXTERNALLY_TIMED_PROGRAMMING = 0x0A
	CMD_ROW_ERASE_PGM = 0x11
	
	delayTinternalProgDM = 0.005
	delayTinternalProgPM = 0.0025
	# overriding: 
	delayTdis = 0.0003
	delayTprog = 0.0021
	delayTdly = 0.0000015
	delayTera = 0.005

	defaultWord = [b'\xFF', b'\x3F']
	nDeviceIdRevisionBits = 5
	
	deviceIdMapDict = {
			0o474:"16F1826", 0o475:"16F1827", 0o504:"16LF1826", 0o505:"16LF1827", 0o471:"16F1823", 0o501:"16LF1823",
			0o470:"16F1822", 0o500:"16LF1822", 0o472:"16F1824", 0o502:"16LF1824", 0o473:"16F1825", 0o503:"16LF1825",
			0o476:"16F1828", 0o506:"16LF1828", 0o477:"16F1829", 0o507:"16LF1829",
			0o515:"10F320", 0o514:"10F322", 0o517:"10LF320", 0o516:"10LF322",
			0o174:"12F629", 0o176:"12F675", 0o206:"16F630", 0o207:"16F676",
			0o053:"16F84A",
			0o546:"12F1501", 0o554:"12LF1501", 0o547:"16F1503", 0o555:"16LF1503", 0o550:"16F1507", 0o556:"16LF1507",
			0o551:"16F1508", 0o557:"16LF1508", 0o552:"16F1509", 0o560:"16LF1509",
			0o460:"16F1933", 0o432:"16F1934", 0o433:"16F1936", 0o434:"16F1937", 0o435:"16F1938", 0o436:"16F1939",
			0o450:"16F1946", 0o451:"16F1947", 0o440:"16LF1933", 0o442:"16LF1934", 0o443:"16LF1936", 0o444:"16LF1937",
			0o445:"16LF1938", 0o446:"16LF1939", 0o454:"16LF1946", 0o455:"16LF1947", 0o541:"16LF1902", 0o540:"16LF1903",
			0o544:"16LF1904", 0o543:"16LF1906", 0o542:"16LF1907",
			0x3020:"16F1454", 0x3024:"16LF1454", 0x3021:"16F1455", 0x3025:"16LF1455", 0x3023:"16F1459", 0x3027:"16LF1459",
			}

	def __init__(self,
	    chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
	    signature,
	    flashPageSize, flashPages,
	    eepromPageSize, eepromPages,
	    fuseBytes
	    ):
		Chip_Microchip8_common.__init__(self, chipPackage, chipPinVCC, chipPinsVPP, chipPinGND, signature, flashPageSize, flashPages, eepromPageSize, eepromPages, fuseBytes)
		self.isInsideProgramMemoryArea = True
		self.initPcValue = 0
		self.userIDLocationAddr = self.logicalFlashProgramMemorySize
		self.deviceIDAddr = self.logicalFlashProgramMemorySize + 0x06
		self.configWordAddr = self.logicalFlashProgramMemorySize + 0x07

	def getIHexInterpreter(self):
		inter = IHexInterpreter()
		inter.progmemRanges = [ AddressRange(0, 2 * self.flashPageSize) ]
		inter.fuseRanges = [ AddressRange(2 * self.configWordAddr,
						  2 * self.configWordAddr + 1) ]
		inter.uilRanges = [ AddressRange(2 * self.userIDLocationAddr,
						 2 * (self.userIDLocationAddr + self.userIDLocationSize) - 1) ]
		inter.progmemDefaultBytes = self.defaultWord[0] + self.defaultWord[1]
		inter.fuseDefaultBytes = self.defaultWord[0] + self.defaultWord[1]
		return inter

	def incrementPC(self, count):
		for address in range(0, count):
			self.sendCommand(0, 0, 0, self.CMD_INCREMENT_ADDRESS)
			self.PC += 1
			if(self.isInsideProgramMemoryArea):
				if (self.PC == self.logicalFlashProgramMemorySize):
					self.PC = 0
			else:
				if (self.PC == self.logicalFlashConfigurationMemorySize):
					self.PC = self.logicalFlashProgramMemorySize

	def enterConfigArea(self, wordLatched=0):
		self.send6bitWriteInstruction(self.CMD_LOAD_CONFIGURATION, wordLatched)
		self.PC = self.logicalFlashProgramMemorySize
		self.isInsideProgramMemoryArea = False
	
	def setPC(self, address):
		if(self.isInsideProgramMemoryArea):
			if(address >= self.logicalFlashProgramMemorySize):
				raise(TOPException('Cannot set PC to address inside PM {:x}'.format(address)))
		else:
			if(address < self.logicalFlashProgramMemorySize):
				raise(TOPException('Cannot set PC to address outside PM {:x}'.format(address)))
		while(self.PC != address):
			self.incrementPC(1)
	
	def readSignature(self):
		self.progressMeterInit("Reading signature", 0)
		self.enterPM()
		self.enterConfigArea()
		self.setPC(self.deviceIDAddr)
		idSize = 1
		for i in range(0, idSize):
			self.sendReadFlashInstr()
			self.top.hostDelay(0.00002)
			self.readSDOBufferLow()
			self.readSDOBufferHigh()
			self.incrementPC(1)
		self.progressMeterFinish()
		signature = self.top.cmdReadBufferReg()[0:2 * idSize]
		devId = ((byte2int(signature[1]) << 8) | byte2int(signature[0])) >> self.nDeviceIdRevisionBits
		if(devId in self.deviceIdMapDict):
			print("device: {:s}".format(self.deviceIdMapDict.get(devId)))
		else:
			print("WARNING: device id {:o} not found in local dictionary".format(devId))
		return signature
			
	def sendWriteFlashInstrExternallyTimed(self):
		'''
		'''
		self.sendCommand(0, 0, 0, self.CMD_BEGIN_EXTERNALLY_TIMED_PROGRAMMING)
		self.top.hostDelay(self.delayTprog)
		self.sendCommand(0, 0, 0, self.CMD_END_EXTERNALLY_TIMED_PROGRAMMING)
		self.top.hostDelay(self.delayTdis)
	
	def sendWriteFlashInstrPM(self):
		'''
		'''
		self.sendCommand(0, 0, 0, self.CMD_BEGIN_INTERNALLY_TIMED_PROGRAMMING)
		self.top.hostDelay(self.delayTinternalProgPM)
	
	def sendWriteFlashInstrCW(self):
		'''
		'''
		self.sendCommand(0, 0, 0, self.CMD_BEGIN_INTERNALLY_TIMED_PROGRAMMING)
		self.top.hostDelay(self.delayTinternalProgDM)	
	
	def sendWriteFlashInstr(self):
		'''
		'''
		self.sendWriteFlashInstrPM()
	
	def sendWriteFlashInstrDM(self):
		'''
		'''
		self.sendWriteFlashInstrExternallyTimed()      



#
# THIS FILE WAS AUTOGENERATED BY makeSip6.py
# Do not edit this file manually. All changes will be lost.
#

"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC24f04ka201 SIP6
#
#    Copyright (c) 2014 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip16_common import *
from configWords import ka200_fuseDesc

class Chip_Pic24f04ka201sip6(Chip_Microchip16_common):
	
	voltageVDD = 3.3
	voltageVPP = 8
	
	logicalFlashProgramMemorySize = 0x800000
	logicalFlashConfigurationMemorySize = 0x800000
	
	hasEEPROM = False
	
	def __init__(self):
	 	Chip_Microchip16_common.__init__(self,
		chipPackage = "DIP10",
		chipPinVCC = 9,
		chipPinsVPP = 10,
		chipPinGND = 8,
		signature="\x00\x0d",
		# flashPageSize (in number of 24bit words)
		flashPageSize=0xAFE / 2 + 2,
		# flashPageSize=0x40,
		flashPages=1,
		# eepromPageSize (in 16bit words)
		eepromPageSize=0,
		eepromPages=0,
		# all 7 words uses lowest byte only
		fuseBytes=2 * 9
		)
		self.configWordAddr = 0xF80000
		# self.osccalBackupAddr = self.userIDLocationAddr + self.userIDLocationSize

fuseDesc = ka200_fuseDesc

ChipDescription(
	Chip_Pic24f04ka201sip6,
	bitfile = "microchip16sip6",
	chipID="pic24f04ka201sip6",
	runtimeID = (0xDF05, 0x01),
	chipVendors="Microchip",
	description = "PIC24F04KA201 - ICD",
	packages = (("DIP10", ""), ),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



"""
#    TOP2049 Open Source programming suite
#
#    M24C16 I2C based serial EEPROM
#
#    Copyright (c) 2011-2017 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *


class Chip_m24cXXdip8_common(Chip):
	I2C_BASE_ADDR	= 0x50 << 1
	I2C_READ	= 0x01
	I2C_WRITE	= 0x00

	def __init__(self, eepromSize):
		Chip.__init__(self,
			      chipPackage = "DIP8",
			      chipPinVCC = 8,
			      chipPinGND = 4)
		self.eepromSize = eepromSize	# in bytes

	def __chipTurnOn(self):
		self.top.cmdSetVCCVoltage(5)
		self.top.cmdSetVPPVoltage(5)
		self.applyVCC(True)
		self.applyVPP(False)
		self.applyGND(True)
		self.top.cmdEnableZifPullups(True)

		self.currentAddrExt = None
		self.currentWriteMode = None

	def erase(self):
		self.writeEEPROM("\xFF" * self.eepromSize)

	def readEEPROM(self):
		self.__chipTurnOn()

		image = ""
		prevAddr = None
		self.progressMeterInit("Reading EEPROM", self.eepromSize)
		for addr in range(0, self.eepromSize):
			self.progressMeter(addr)
			if prevAddr is None or (prevAddr & 0xFF00) != (addr & 0xFF00):
				self.__setAddressExtension(addr, writeMode=False)
				# Begin sequential random read
				self.__runI2C(data=self.I2C_BASE_ADDR | self.currentAddrExt | self.I2C_WRITE,
					      read=False, do_start=True, do_stop=False)
				self.__expectACK()
				self.__runI2C(data=addr & 0xFF,
					      read=False, do_start=False, do_stop=False)
				self.__expectACK()
				self.__runI2C(data=self.I2C_BASE_ADDR | self.currentAddrExt | self.I2C_READ,
					      read=False, do_start=True, do_stop=False)
				self.__expectACK()
				prevAddr = addr
			# Sequential random read
			if addr >= self.eepromSize - 1:
				# Last byte
				self.__runI2C(read=True, do_start=False, do_stop=True)
				self.__expectNACK()
			else:
				self.__runI2C(read=True, do_start=False, do_stop=False,
					      drive_ack=True)
				self.__expectACK()
			self.__readData()
			image += self.top.cmdReadBufferReg(1)
		self.progressMeterFinish()

		return image

	def writeEEPROM(self, image):
		if len(image) > self.eepromSize:
			self.throwError("Invalid EEPROM image size %d (expected <=%d)" %\
				(len(image), self.eepromSize))
		self.__chipTurnOn()

		self.progressMeterInit("Writing EEPROM", len(image))
		prevAddr = None
		for addr in range(0, len(image)):
			self.progressMeter(addr)
			if prevAddr is None or (prevAddr & 0xFFF0) != (addr & 0xFFF0):
				self.__setAddressExtension(addr, writeMode=True)
				self.__runI2C(data=self.I2C_BASE_ADDR | self.currentAddrExt | self.I2C_WRITE,
					      read=False, do_start=True, do_stop=False)
				self.__expectACK()
				self.__runI2C(data=addr & 0xFF,
					      read=False, do_start=False, do_stop=False)
				self.__expectACK()
				prevAddr = addr
			if (addr & 0xF) == 0xF:
				self.__runI2C(data=byte2int(image[addr]),
					      read=False, do_start=False, do_stop=True)
				self.__expectACK()
				self.top.cmdDelay(0.005) # Max write time
			else:
				self.__runI2C(data=byte2int(image[addr]),
					      read=False, do_start=False, do_stop=False)
				self.__expectACK()

		self.progressMeterFinish()

	def __readData(self):
		self.top.cmdFPGARead(0)

	def __setData(self, dataByte):
		self.top.cmdFPGAWrite(1, dataByte & 0xFF)

	def __setAddressExtension(self, address, writeMode):
		sizeMask = self.eepromSize - 1
		assert(sizeMask & ~0x7FF == 0)
		addrExt = ((address & 0x700 & sizeMask) >> 8) << 1

		if self.currentWriteMode != writeMode:
			E0 = E1 = E2 = 0
			E0_en = not (sizeMask & 0x0100)
			E1_en = not (sizeMask & 0x0200)
			E2_en = not (sizeMask & 0x0400)
			WC = not writeMode
			self.__setControlPins(E0=E0, E0_en=E0_en,
					      E1=E1, E1_en=E1_en,
					      E2=E2, E2_en=E2_en,
					      WC=WC)
		self.currentAddrExt = addrExt
		self.currentWriteMode = writeMode

	def __runI2C(self, data=None, read=False, do_start=False, do_stop=False, drive_ack=False):
		if data is not None:
			self.__setData(data)
		else:
			self.__setData(0)
		command = (0x01 if read else 0) |\
			  (0x02 if do_start else 0) |\
			  (0x04 if do_stop else 0) |\
			  (0x08 if drive_ack else 0)
		self.top.cmdFPGAWrite(0, command)
		self.__busyWait()

	def __isBusy(self):
		(busy, ack) = self.__getStatusFlags()
		return busy

	def __busyWait(self):
		for i in range(0, 100):
			if not self.__isBusy():
				return
			self.top.hostDelay(0.001)
		self.throwError("Timeout in busywait.")

	def __getStatusFlags(self):
		self.top.cmdFPGARead(1)
		stat = self.top.cmdReadBufferReg8()
		busy = bool(stat & 0x01)
		ack = not bool(stat & 0x02)
		self.lastAck = ack
		return (busy, ack)

	def __expectACK(self):
		if not self.lastAck:
			self.throwError("Expected I2C ACK, but received NACK")

	def __expectNACK(self):
		if self.lastAck:
			self.throwError("Expected I2C NACK, but received ACK")

	def __setControlPins(self, E0, E0_en, E1, E1_en, E2, E2_en, WC):
		value = 0
		if E0:
			value |= (1 << 0)
		if E0_en:
			value |= (1 << 1)
		if E1:
			value |= (1 << 2)
		if E1_en:
			value |= (1 << 3)
		if E2:
			value |= (1 << 4)
		if E2_en:
			value |= (1 << 5)
		if WC:
			value |= (1 << 6)
		self.top.cmdFPGAWrite(2, value)

class Chip_m24c01dip8(Chip_m24cXXdip8_common):
	def __init__(self):
		Chip_m24cXXdip8_common.__init__(self, eepromSize = 1024 * 1 // 8)

class Chip_m24c02dip8(Chip_m24cXXdip8_common):
	def __init__(self):
		Chip_m24cXXdip8_common.__init__(self, eepromSize = 1024 * 2 // 8)

class Chip_m24c04dip8(Chip_m24cXXdip8_common):
	def __init__(self):
		Chip_m24cXXdip8_common.__init__(self, eepromSize = 1024 * 4 // 8)

class Chip_m24c08dip8(Chip_m24cXXdip8_common):
	def __init__(self):
		Chip_m24cXXdip8_common.__init__(self, eepromSize = 1024 * 8 // 8)

class Chip_m24c16dip8(Chip_m24cXXdip8_common):
	def __init__(self):
		Chip_m24cXXdip8_common.__init__(self, eepromSize = 1024 * 16 // 8)

class ChipDescription_m24cXX(ChipDescription):
	def __init__(self, chipImplClass, chipID, description):
		ChipDescription.__init__(self,
			chipImplClass = chipImplClass,
			bitfile = "m24c16dip8",
			chipID = chipID,
			runtimeID = (0x000B, 0x01),
			chipType = ChipDescription.TYPE_EEPROM,
			chipVendors = "ST",
			description = description,
			packages = (
				("DIP8", ""),
				("SO8", "With 1:1 adapter"),
				("TSSOP8", "With 1:1 adapter"),
			),
		)

ChipDescription_m24cXX(
	Chip_m24c01dip8,
	chipID = "m24c01dip8",
	description = "M24C01 I2C EEPROM",
)

ChipDescription_m24cXX(
	Chip_m24c02dip8,
	chipID = "m24c02dip8",
	description = "M24C02 I2C EEPROM",
)

ChipDescription_m24cXX(
	Chip_m24c04dip8,
	chipID = "m24c04dip8",
	description = "M24C04 I2C EEPROM",
)

ChipDescription_m24cXX(
	Chip_m24c08dip8,
	chipID = "m24c08dip8",
	description = "M24C08 I2C EEPROM",
)

ChipDescription_m24cXX(
	Chip_m24c16dip8,
	chipID = "m24c16dip8",
	description = "M24C16 I2C EEPROM",
)



"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC18F67J60 SIP6
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_18f97j60family import *

class Chip_PIC18F67J60sip6(microchip8_18f97j60family):

	hasEEPROM = True

     	writeBufferSize			 = 8
     	eraseBufferSize			 = 64
     	
     	def __init__(self):
     		microchip8_18f97j60family.__init__(self,
			chipPackage="DIP10",
			chipPinVCC=9,
			chipPinsVPP=10,
			chipPinGND=8,
			signature="\x22\x21",
			flashPageSize=0x2000,
			flashPages=1,
			eepromPageSize=0x100,
			eepromPages=1,
			fuseBytes=6
			)
			

fuseDesc = (
	BitDescription(0o00, "WDTEN"),
	BitDescription(0o01, "NA"),
	BitDescription(0o02, "NA"),
	BitDescription(0o03, "NA"),
	BitDescription(0o04, "NA"),
	BitDescription(0o05, "STVREN"),
	BitDescription(0o06, "XINST"),
	BitDescription(0o07, "nDEBUG"),
	BitDescription(0o10, "NA"),
	BitDescription(0o11, "NA"),
	BitDescription(0o12, "CP[0]"),
	BitDescription(0o13, "NA"),
	BitDescription(0o14, "NA"),
	BitDescription(0o15, "NA"),
	BitDescription(0o16, "NA"),
	BitDescription(0o17, "NA"),
    
	BitDescription(0o20, "FOSC[0]"),
	BitDescription(0o21, "FOSC[1]"),
	BitDescription(0o22, "FOSC[2]"),
	BitDescription(0o23, "NA"),
	BitDescription(0o24, "NA"),
	BitDescription(0o25, "NA"),
	BitDescription(0o26, "FCMEN, 0=Fail-Safe Clock Monitor is disabled"),
	BitDescription(0o27, "IESO, 0=Internal/External Switchover mode is disabled"),
	BitDescription(0o30, "WDTPS[0]"),
	BitDescription(0o31, "WDTPS[1]"),
	BitDescription(0o32, "WDTPS[2]"),
	BitDescription(0o33, "WDTPS[3]"),
	BitDescription(0o34, "NA"),
	BitDescription(0o35, "NA"),
	BitDescription(0o36, "NA"),
	BitDescription(0o37, "NA"),
	
	BitDescription(0o40, "NA"),
	BitDescription(0o41, "NA"),
	BitDescription(0o42, "NA"),
	BitDescription(0o43, "NA"),
	BitDescription(0o44, "NA"),
	BitDescription(0o45, "NA"),
	BitDescription(0o46, "NA"),
	BitDescription(0o47, "NA"),
	BitDescription(0o50, "NA"),
	BitDescription(0o51, "NA"),
	BitDescription(0o52, "NA"),
	BitDescription(0o53, "NA"),
	BitDescription(0o54, "NA"),
	BitDescription(0o55, "ETHLED"),
	BitDescription(0o56, "NA"),
	BitDescription(0o57, "NA"),
)

ChipDescription(
	Chip_PIC18F67J60sip6,
	bitfile="microchip01sip6",
	chipID="PIC18F67J60sip6",
	runtimeID=(0xDE05, 0x01),
	chipVendors="Microchip",
	description="PIC18F67J60",
	packages=(("sip6", ""),),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



#
# THIS FILE WAS AUTOGENERATED BY makeSip6.py
# Do not edit this file manually. All changes will be lost.
#

"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC16F59 SIP6
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_singlePMarea import *


class Chip_Pic16F59sip6(microchip8_singlePMarea):

    logicalFlashSize = 0x1000
    

    def __init__(self):
	microchip8_singlePMarea.__init__(self,
		  chipPackage = "DIP10",
		  chipPinVCC = 9,
		  chipPinsVPP = 10,
		  chipPinGND = 8,
		  signature="",
		  flashPageSize=0x200,
		  flashPages=4,
		  eepromPageSize=0,
		  eepromPages=0,
		  fuseBytes=2
		  )
    
fuseDesc = (
	BitDescription(0, "FOSC0"),
	BitDescription(1, "FOSC1 - 00=LP, 01=XT, 10=HS, 11=RC"),
	BitDescription(2, "WDTE"),
	BitDescription(3, "!CP"),
	BitDescription(4, "Unused"),
	BitDescription(5, "Unused"),
	BitDescription(6, "Unused"),
	BitDescription(7, "Unused"),
	BitDescription(8, "Unused"),
	BitDescription(9, "Unused"),
	BitDescription(10, "Unused"),
	BitDescription(11, "Unused"),
)

ChipDescription(
	Chip_Pic16F59sip6,
	bitfile = "microchip01sip6",
	chipID="pic16f59sip6",
	runtimeID = (0xDE05, 0x01),
	chipVendors="Microchip",
	description = "PIC16F59 - ICD",
	packages = (("DIP10", ""), ),
	fuseDesc=fuseDesc,
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



"""
#    *.BIT file parser
#
#    Copyright (c) 2009-2011 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
try:
	import pkg_resources
except ImportError:
	print "Failed to import the 'pkg_resources' Python module."
	print "'pkg_resources' is part of the Python 'setuptools' package."
	print "On Debian Linux run:  apt-get install python-pkg-resources"
	sys.exit(1)

from util import *


class BitfileException(Exception): pass

class Bitfile:
	# Magic header
	MAGIC		= "\x00\x09\x0f\xf0\x0f\xf0\x0f\xf0\x0f\xf0\x00\x00\x01"
	# Field IDs
	FIELD_SRCFILE	= 0x61
	FIELD_FPGA	= 0x62
	FIELD_DATE	= 0x63
	FIELD_TIME	= 0x64
	FIELD_PAYLOAD	= 0x65

	def __init__(self):
		self.filename = ""
		self.srcfile = ""
		self.fpga = ""
		self.date = ""
		self.time = ""
		self.payload = ""

	def getFilename(self):
		return self.filename

	def getSrcFile(self):
		return self.srcfile

	def getFPGA(self):
		return self.fpga

	def getDate(self):
		return self.date

	def getTime(self):
		return self.time

	def getPayload(self):
		return self.payload

	def parseFile(self, filename):
		try:
			fd = open(filename, "rb")
			data = fd.read()
			fd.close()
		except (IOError), e:
			raise BitfileException("Failed to read \"" + filename + "\": " + e.strerror)
		self.filename = filename
		self.__parse(data)

	def __parse(self, data):
		try:
			magic = data[0:len(self.MAGIC)]
			if magic != self.MAGIC:
				raise BitfileException("Invalid magic header")
			i = len(self.MAGIC)
			while i < len(data):
				i += self.__parseNextField(data, i)
		except (IndexError), e:
			raise BitfileException("Failed to parse BIT file")
		if not self.fpga:
			raise BitfileException("No FPGA ID string found")
		if not self.payload:
			raise BitfileException("No payload found")

	def __parseNextField(self, data, i):
		fieldId = byte2int(data[i + 0])
		if (fieldId == self.FIELD_SRCFILE):
			data = self.__parse16bitField(data, i + 1)
			self.srcfile = data.decode("ASCII").strip().strip("\x00")
			return len(data) + 3
		if (fieldId == self.FIELD_FPGA):
			data = self.__parse16bitField(data, i + 1)
			self.fpga = data.decode("ASCII").strip().strip("\x00")
			return len(data) + 3
		if (fieldId == self.FIELD_DATE):
			data = self.__parse16bitField(data, i + 1)
			self.date = data.decode("ASCII").strip().strip("\x00")
			return len(data) + 3
		if (fieldId == self.FIELD_TIME):
			data = self.__parse16bitField(data, i + 1)
			self.time = data.decode("ASCII").strip().strip("\x00")
			return len(data) + 3
		if (fieldId == self.FIELD_PAYLOAD):
			self.payload = self.__parse32bitField(data, i + 1)
			return len(self.payload) + 5
		raise BitfileException("Found unknown data field 0x%02X" % fieldId)

	def __parse16bitField(self, data, i):
		fieldLen = (byte2int(data[i + 0]) << 8) | byte2int(data[i + 1])
		return data[i + 2 : i + 2 + fieldLen]

	def __parse32bitField(self, data, i):
		fieldLen = (byte2int(data[i + 0]) << 24) | (byte2int(data[i + 1]) << 16) |\
			   (byte2int(data[i + 2]) << 8) | byte2int(data[i + 3])
		return data[i + 4 : i + 4 + fieldLen]

def __probeFile(fullpath):
	try:
		open(fullpath, "rb").close()
	except (IOError), e:
		return False
	return True

def bitfileFind(filename):
	"Search some standard paths for a bitfile"
	if not filename.endswith(".bit"):
		filename += ".bit"
	if filename.startswith("/"):
		if __probeFile(filename):
			return filename
	paths = ( ".", "./libtoprammer/fpga/bin", )
	for path in paths:
		fullpath = path + "/" + filename
		if __probeFile(fullpath):
			return fullpath
	fullpath = pkg_resources.resource_filename("libtoprammer",
						   "fpga/bin/" + filename)
	if __probeFile(fullpath):
		return fullpath
	return None

if __name__ == "__main__":
	if len(sys.argv) != 3:
		print "Usage: %s file.bit ACTION"
		print ""
		print "Actions:"
		print " GETSRC       - print the src-file field to stdout"
		print " GETFPGA      - print the fpga-type field to stdout"
		print " GETDATE      - print the date field to stdout"
		print " GETTIME      - print the time field to stdout"
		print " GETPAYLOAD   - print the payload field to stdout"
		sys.exit(1)
	filename = sys.argv[1]
	action = sys.argv[2].upper()

	b = Bitfile()
	b.parseFile(filename)
	if action == "GETSRC":
		sys.stdout.write(b.getSrcFile())
	if action == "GETFPGA":
		sys.stdout.write(b.getFPGA())
	if action == "GETDATE":
		sys.stdout.write(b.getDate())
	if action == "GETTIME":
		sys.stdout.write(b.getTime())
	if action == "GETPAYLOAD":
		sys.stdout.write(b.getPayload())



"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC10F200, PIC10F204 and PIC10f220 DIP8
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_singlePMarea import *

class Chip_Pic10F200dip8(microchip8_singlePMarea):
    
    # CONFIGURATION WORD FOR PIC10F200/202/204/206
    # X X X X   X X X MCLRE     /CP WDT X X
    logicalFlashSize = 0x200

    def __init__(self):
	microchip8_singlePMarea.__init__(self,
		  chipPackage="DIP8",
		  chipPinVCC=2,
		  chipPinsVPP=8,
		  chipPinGND=7,
		  signature="\x09\x18\x24\x35",
		  flashPageSize=0x100,
		  flashPages=1,
		  eepromPageSize=0,
		  eepromPages=0,
		  fuseBytes=2
		  )
	
    
fuseDesc = (
	BitDescription(0, "Unused"),
	BitDescription(1, "Unused"),
	BitDescription(2, "WDTE"),
	BitDescription(3, "!CP"),
	BitDescription(4, "MCLRE"),
	BitDescription(5, "Unused"),
	BitDescription(6, "Unused"),
	BitDescription(7, "Unused"),
	BitDescription(8, "Unused"),
	BitDescription(9, "Unused"),
	BitDescription(10, "Unused"),
	BitDescription(11, "Unused"),
)

ChipDescription(
	Chip_Pic10F200dip8,
	bitfile="pic10fxxxdip8",
	chipID="pic10f200dip8",
	runtimeID=(0xDE01, 0x01),
	chipVendors="Microchip",
	description="PIC10F200, PIC10F204, PIC10F220",
	packages=(("DIP8", ""),),
	fuseDesc=fuseDesc,
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



"""
#    TOP2049 Open Source programming suite
#
#    Microchip8 - 18f97j60 family - 8bit PIC MCU
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chips.microchip8.microchip8_18_common import *

class microchip8_18f97j60family(Chip_Microchip8_18_common):
	
	voltageVDD				 = 3
	voltageVPP				 = 3
	
	delayP5A = 0.000000040  # Delay between 4-bit command operand and next 4-bit command
	delayP9 = 0.0034  # SCK High time (minimum programming time)
	delayP10 = 0.000005  # SCK Low time after programming (high-voltage discharge time)
	delayP11 = 0.475  # Delay to allow self-timed data write or bulk erase to occur
	delayP12 = 0.0004  # Input data hold time from nMCLR/Vpp rise
	delayP19 = 0.000001 # Delay from first nMCLR fall to first PGC rise for key sequence on PGD
	delayP20 = 0.00000004 # Delay from last PGC fall for key sequence on PGD to second nMCLR rise
	
	def __init__(self,
			chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes
			):
		Chip_Microchip8_18_common.__init__(self,
		chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
			signature,
			flashPageSize, flashPages,
			eepromPageSize, eepromPages,
			fuseBytes)
		
	def enterPM(self, force=False):
		if self.isInPmMode and not force:
			return
		"Enter HV programming mode. Vdd first entry mode"
		self.applyVCC(False)
		self.applyVPP(False)
		self.applyGND(False)
		self.setPins(0, 0)
		#self.top.cmdSetVCCVoltage(self.voltageVDD)
		self.top.cmdSetVPPVoltage(self.voltageVPP)
		self.applyGND(True)
		self.applyVCC(True)
		self.top.hostDelay(self.delayP13)
		self.applyVPP(True)
		self.applyVPP(False)
		self.top.hostDelay(self.delayP19)
		self.setTopProgrammerDelays()
		#program entry code
		#print("sending enterpm")
		self.top.cmdFPGAWrite(0x18,0xFF)
		self.top.hostDelay(self.delayP20)
		self.applyVPP(True)
		self.top.hostDelay(self.delayP12)
		
		self.isInPmMode = True
		
	def erase(self):
		self.progressMeterInit("Erasing chip", 0)
		self.enterPM(True)
		self.executeCode(self.getCodeAddrToTBLPTR(0x3C0004))
		self.send4bitWriteInstruction(self.CMD_TW, 0x0080)
		self.executeCode((0x0000,))
		self.sendCommand(1)		
		self.top.hostDelay(self.delayP11 + self.delayP10)
		for i in range(0,4):
			self.sendCommand(1)
		self.top.flushCommands()
		self.progressMeterFinish()

	def writeSequentialBlock(self, startAddr, image, size, infoText):
		if len(image) > size:
			self.throwError("Invalid flash image size %d (expected <=%d)" % \
				(len(image), self.userIDLocationSize))
		self.enterPM()
		self.executeCode((0x8EA6, 0x9CA6))
		self.progressMeterInit(infoText, len(image) / 8)
		self.executeCode(self.getCodeAddrToTBLPTR(startAddr))
		for blockAddr in range(0, len(image), 8):
			self.write8bytes(image[blockAddr:])
			self.progressMeter(blockAddr)
		self.progressMeterFinish()
		self.exitPM()		


"""
#    TOP2049 Open Source programming suite
#
#    Generic SRAM chip
#
#    Copyright (c) 2011 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chip import *


class Chip_genericSRAM(Chip):
	def __init__(self, chipPackage, chipPinVCC, chipPinGND,
		     VCCVoltage,
		     nrAddressBits, nrDataBits):
		Chip.__init__(self,
			      chipPackage = chipPackage,
			      chipPinVCC = chipPinVCC,
			      chipPinGND = chipPinGND)
		self.VCCVoltage = VCCVoltage
		self.nrAddressBits = nrAddressBits
		self.nrAddressBytes = int(math.ceil((float(self.nrAddressBits) - 0.1) / 8))
		self.nrDataBits = nrDataBits
		assert(nrDataBits == 8)

	def erase(self):
		self.writeRAM(int2byte(0) * self.__sizeBytes())

	def test(self):
		generic = GenericAlgorithms(self)
		generic.simpleTest(self.readRAM, self.writeRAM,
				   self.__sizeBytes())

	def readRAM(self):
		image = []

		self.progressMeterInit("Reading SRAM", self.__sizeBytes())
		self.__turnOnChip()
		self.__setControlPins(CE=0, OE=0, WE=1)
		nrBytes = 0
		for addr in range(0, self.__sizeBytes()):
			self.progressMeter(addr)
			self.__setAddress(addr)
			self.__readData()
			nrBytes += 1
			if nrBytes == self.top.getBufferRegSize():
				image.append(self.top.cmdReadBufferReg(nrBytes))
				nrBytes = 0
		image.append(self.top.cmdReadBufferReg(nrBytes))
		self.__setControlPins(CE=1, OE=1, WE=1)
		self.progressMeterFinish()

		return b"".join(image)

	def writeRAM(self, image):
		if len(image) > self.__sizeBytes():
			self.throwError("Invalid memory image size %d (expected max %d)" %\
				(len(image), self.__sizeBytes()))

		self.progressMeterInit("Writing SRAM", self.__sizeBytes())
		self.__turnOnChip()
		self.__setControlPins(CE=0, OE=1, WE=1)
		for addr in range(0, len(image)):
			self.progressMeter(addr)
			self.__setAddress(addr)
			self.__writeData(image[addr])
			self.__setControlPins(CE=0, OE=1, WE=0)
			self.top.cmdDelay(0.00000007) # Delay at least 70 nsec
			self.__setControlPins(CE=0, OE=1, WE=1)
		self.__setControlPins(CE=1, OE=1, WE=1)
		self.progressMeterFinish()

	def __sizeBytes(self):
		return (1 << self.nrAddressBits)

	def __turnOnChip(self):
		self.__setControlPins(CE=1, OE=1, WE=1)
		self.top.cmdSetVCCVoltage(self.VCCVoltage)
		self.applyGND(True)
		self.applyVCC(True)
		self.lastAddress = None

	def __setControlPins(self, CE=1, OE=1, WE=1):
		value = 0
		if CE:
			value |= 1
		if OE:
			value |= 2
		if WE:
			value |= 4
		self.top.cmdFPGAWrite(0x11, value)

	def __writeData(self, data):
		data = byte2int(data)
		self.top.cmdFPGAWrite(0x10, data)

	def __readData(self):
		self.top.cmdFPGARead(0x10)

	def __setAddress(self, addr):
		for i in range(0, self.nrAddressBytes):
			shift = 8 * i
			mask = 0xFF << shift
			if self.lastAddress is None or\
			   (self.lastAddress & mask) != (addr & mask):
				self.top.cmdFPGAWrite(0x12 + i,
						      (addr & mask) >> shift)
		self.lastAddress = addr



#
# THIS FILE WAS AUTOGENERATED BY makeSip6.py
# Do not edit this file manually. All changes will be lost.
#

"""
#    TOP2049 Open Source programming suite
#
#   Microchip PIC10F202, PIC10F206 and PIC10f222 SIP6
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from microchip8_singlePMarea import *

class Chip_Pic10F202sip6(microchip8_singlePMarea):

	# CONFIGURATION WORD FOR PIC10F200/202/204/206
	# X X X X   X X X MCLRE     /CP WDT X X
	logicalFlashSize = 0x400

	def __init__(self):
		microchip8_singlePMarea.__init__(self,
			chipPackage = "DIP10",
			chipPinVCC = 9,
			chipPinsVPP = 10,
			chipPinGND = 8,
			signature="\x09\x18\x24\x35",
			flashPageSize=0x200,
			flashPages=1,
			eepromPageSize=0,
			eepromPages=0,
			fuseBytes=2
			)
 

fuseDesc = (
	BitDescription(0, "Unused"),
	BitDescription(1, "Unused"),
	BitDescription(2, "WDTE"),
	BitDescription(3, "!CP"),
	BitDescription(4, "MCLRE"),
	BitDescription(5, "Unused"),
	BitDescription(6, "Unused"),
	BitDescription(7, "Unused"),
	BitDescription(8, "Unused"),
	BitDescription(9, "Unused"),
	BitDescription(10, "Unused"),
	BitDescription(11, "Unused"),
)

ChipDescription(
	Chip_Pic10F202sip6,
	bitfile = "microchip01sip6",
	chipID="pic10f202sip6",
	runtimeID = (0xDE05, 0x01),
	chipVendors="Microchip",
	description = "PIC10F202, PIC10F206, PIC10F222 - ICD",
	packages = (("DIP10", ""), ),
	fuseDesc=fuseDesc, 	
	maintainer="Pavel Stemberk <stemberk@gmail.com>",
)



#!/usr/bin/env python

from distutils.core import setup
import libtoprammer.main as toprammer_main

setup(	name		= "toprammer",
	version		= toprammer_main.VERSION,
	description	= "TOP2049 Open Source programming suite",
	long_description = "Toprammer is an Opensource software for the TOP2049 universal programmer.",
	author		= "Michael Buesch",
	author_email	= "m@bues.ch",
	url		= "http://bues.ch/h/toprammer",
	packages	= [ "libtoprammer",
			    "libtoprammer/top2049",
			    "libtoprammer/chips",
			    "libtoprammer/chips/microchip8",
			    "libtoprammer/chips/microchip16", ],
	package_data	= { "libtoprammer" : [ "fpga/bin/*.bit",
					       "icons/*.png", ], },
	scripts		= [ "toprammer", "toprammer-gui", "toprammer-layout", ],
	keywords	= [ "universal programmer", "TOP", "TOP2049", "TOP2007",
			    "TOP3000", "TOP3100", "TOP853", ],
	classifiers	= [
		"Development Status :: 5 - Production/Stable",
		"Environment :: Console",
		"Environment :: X11 Applications",
		"Environment :: X11 Applications :: Qt",
		"Intended Audience :: Developers",
		"Intended Audience :: Education",
		"Intended Audience :: Information Technology",
		"Intended Audience :: Manufacturing",
		"Intended Audience :: Science/Research",
		"License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
		"Operating System :: POSIX",
		"Operating System :: POSIX :: Linux",
		"Programming Language :: Python",
		"Programming Language :: Python :: 2.7",
		"Programming Language :: Python :: Implementation :: CPython",
		"Topic :: Education",
		"Topic :: Scientific/Engineering",
		"Topic :: Software Development",
		"Topic :: Software Development :: Embedded Systems",
		"Topic :: Software Development :: Testing",
	],
)



"""
#    TOP2049 Open Source programming suite
#
#    Atmel Mega88 DIP28 support
#
#    Copyright (c) 2009-2010 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from atmega_common import *


class Chip_ATMega88DIP28(Chip_ATMega_common):
	def __init__(self):
		Chip_ATMega_common.__init__(self,
			chipPackage = "DIP28",
			chipPinVCC = 7,
			chipPinsVPP = 1,
			chipPinGND = 8,
			signature = "\x1E\x93\x0A",
			flashPageSize = 32,
			flashPages = 128,
			eepromPageSize = 4,
			eepromPages = 128,
			fuseBytes = 3)

ChipDescription(
	Chip_ATMega88DIP28,
	bitfile = "atmega8dip28",
	chipID = "atmega88dip28",
	runtimeID = (0x0003, 0x01),
	chipVendors = "Atmel",
	description = "AtMega88",
	packages = ( ("DIP28", ""), ),
)

class Chip_ATMega88paDIP28(Chip_ATMega_common):
	def __init__(self):
		Chip_ATMega_common.__init__(self,
			chipPackage = "DIP28",
			chipPinVCC = 7,
			chipPinsVPP = 1,
			chipPinGND = 8,
			signature = "\x1E\x93\x0F",
			flashPageSize = 32,
			flashPages = 128,
			eepromPageSize = 4,
			eepromPages = 128,
			fuseBytes = 3)

ChipDescription(
	Chip_ATMega88paDIP28,
	bitfile = "atmega8dip28",
	chipID = "atmega88padip28",
	runtimeID = (0x0003, 0x01),
	chipVendors = "Atmel",
	description = "AtMega88PA",
	packages = ( ("DIP28", ""), ),
)



"""
#    TOP2049 Open Source programming suite
#
#    pic8_splittedPMarea - file for newer 8bit PIC MCUs
#
#    Copyright (c) 2013 Pavel Stemberk <stemberk@gmail.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from libtoprammer.chips.microchip8.microchip8_splittedPMarea import *

class microchip8_splittedPMarea_hasResetPC(microchip8_splittedPMarea):
	
    CMD_RESET_ADDRESS	 = 0x16
    
    userIDLocationSize = 4
    voltageVDD = 3
    voltageVPP = 8.5
    
    logicalFlashProgramMemorySize = 0x8000
    logicalFlashConfigurationMemorySize = 0x8000    
    
    def __init__(self,
	    chipPackage, chipPinVCC, chipPinsVPP, chipPinGND,
	    signature,
	    flashPageSize, flashPages,
	    eepromPageSize, eepromPages,
	    fuseBytes
	    ):
	microchip8_splittedPMarea.__init__(self, chipPackage, chipPinVCC, chipPinsVPP, chipPinGND, signature, flashPageSize, flashPages, eepromPageSize, eepromPages, fuseBytes)

    def setPC(self, address):
	if(self.isInsideProgramMemoryArea):
	    if(address >= self.logicalFlashProgramMemorySize):
		raise(TOPException('Cannot set PC to address inside PM {:x}'.format(address)))
	    if(address < self.PC):
		self.resetPC()
		self.setPC(address)
	else:
	    if(address < self.logicalFlashProgramMemorySize):
		raise(TOPException('Cannot set PC to address outside PM {:x}'.format(address)))
	    if(address < self.PC):
		self.resetPC()
		self.enterConfigArea()
		self.setPC(address)
	while(self.PC != address):
	    self.incrementPC(1)
		    
    def resetPC(self):
	if hasattr(self, 'osccalAddr'):
	    if not hasattr(self, 'CMD_RESET_ADDRESS'):
		print("reset instruction is not supported")
		self.exitPM()
		self.enterPM()
	else:
	    self.sendCommand(0, 0, 0, self.CMD_RESET_ADDRESS)
	self.PC = 0
	self.isInsideProgramMemoryArea = True
    



"""
#    TOP2049 Open Source programming suite
#
#    Utility functions
#
#    Copyright (c) 2009-2011 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
import re
import math
import random


class TOPException(Exception): pass


if sys.version_info[0] == 2: # Python 2.x
	def byte2int(byte):
		return ord(byte)

	def int2byte(integer):
		return chr(integer)
else: # Python 3.x
	def byte2int(byte):
		return int(byte[0])

	def int2byte(integer):
		return bytes( (integer, ) )

def hex2bin(hexdata):
	assert(len(hexdata) % 2 == 0)
	bindata = map(lambda i: int2byte(int(hexdata[i:i+2], 16)),
		      range(0, len(hexdata), 2))
	return b"".join(bindata)

def byte2hex(byte):
	return "%02X" % byte2int(byte)

def bytes2hex(bindata):
	if not bindata:
		return ""
	return "".join(map(byte2hex, bindata))

def byte2ascii(c):
	ci = byte2int(c)
	if ci >= 32 and ci <= 126:
		return c.decode("ASCII")
	return "."

def bytes2ascii(bindata):
	if not bindata:
		return ""
	return "".join(map(byte2ascii, bindata))

def str2bool(string):
	string = str(string).lower().strip()
	if string in ("false", "off", "no"):
		return False
	if string in ("true", "on", "yes"):
		return True
	try:
		return bool(int(string, 10))
	except (ValueError), e:
		pass
	return None

def genRandomBlob(size):
	blob = map(lambda x: int2byte(random.randint(0, 0xFF)),
		   range(0, size))
	return b"".join(blob)

def bit(bitNr):
	return 1 << bitNr

def nrBitsSet(integer):
	count = 0
	while integer:
		count += (integer & 1)
		integer >>= 1
	return count

def roundup(x, y):
	x, y = int(x), int(y)
	return ((x + (y - 1)) // y) * y

hexdump_re = re.compile(r"0x[0-9a-fA-F]+:\s+([0-9a-fA-F\s]+)\s*.*")

def parseHexdump(dump):
	try:
		bin = []
		for line in dump.splitlines():
			line = line.strip()
			if not line:
				continue
			m = hexdump_re.match(line)
			if not m:
				raise TOPException("Invalid hexdump format (regex failure)")
			bytes = m.group(1).replace(" ", "")
			if len(bytes) % 2 != 0:
				raise TOPException("Invalid hexdump format (odd bytestring len)")
			for i in range(0, len(bytes), 2):
				byte = int(bytes[i:i+2], 16)
				bin.append(int2byte(byte))
		return b"".join(bin)
	except (ValueError), e:
		raise TOPException("Invalid hexdump format (Integer error)")

def generateHexdump(mem):
	ret = ""
	asc = ""
	for i in range(0, len(mem)):
		if i % 16 == 0 and i != 0:
			ret += "  " + asc + "\n"
			asc = ""
		if i % 16 == 0:
			ret += "0x%04X:  " % i
		c = byte2int(mem[i])
		ret += "%02X" % c
		if (i % 2 != 0):
			ret += " "
		asc += byte2ascii(mem[i])
	ret += "  " + asc + "\n\n"
	return ret

def dumpMem(mem):
	sys.stdout.write(generateHexdump(mem))

class IO_ihex(object):
	TYPE_DATA = 0
	TYPE_EOF  = 1
	TYPE_ESAR = 2
	TYPE_SSAR = 3
	TYPE_ELAR = 4
	TYPE_SLAR = 5

	def autodetect(self, data):
		try:
			self.toBinary(data)
		except (TOPException), e:
			return False
		return True

	def toBinary(self, ihexData, addressRange=None, defaultBytes=b"\xFF"):
		bin = []
		checksumWarned = False
		doublewriteWarned = False
		addrBias = addressRange.startAddress if addressRange else 0
		try:
			lines = ihexData.splitlines()
			hiAddr = 0
			segment = 0
			for line in lines:
				line = line.strip()
				if len(line) == 0:
					continue
				if len(line) < 11 or (len(line) - 1) % 2 != 0:
					raise TOPException("Invalid IHEX format (length error)")
				if line[0] != ':':
					raise TOPException("Invalid IHEX format (magic error)")
				count = int(line[1:3], 16)
				if len(line) != count * 2 + 11:
					raise TOPException("Invalid IHEX format (count error)")
				addr = (int(line[3:5], 16) << 8) | int(line[5:7], 16)
				addr |= hiAddr << 16
				addr += segment * 16
				if hiAddr and segment:
					print("WARNING: IHEX has ESAR and ELAR record")
				type = int(line[7:9], 16)
				checksum = 0
				for i in range(1, len(line), 2):
					byte = int(line[i:i+2], 16)
					checksum = (checksum + byte) & 0xFF
				checksum = checksum & 0xFF
				if checksum != 0 and not checksumWarned:
					checksumWarned = True
					print "WARNING: Invalid IHEX format (checksum error)"

				if type == self.TYPE_EOF:
					break
				if type == self.TYPE_ESAR:
					if count != 2:
						raise TOPException("Invalid IHEX format (inval ESAR)")
					segment = (int(line[9:11], 16) << 8) | int(line[11:13], 16)
					continue
				if type == self.TYPE_ELAR:
					if count != 2:
						raise TOPException("Invalid IHEX format (inval ELAR)")
					hiAddr = (int(line[9:11], 16) << 8) | int(line[11:13], 16)
					continue
				if addressRange and addr < addressRange.startAddress:
					continue
				if addressRange and addr > addressRange.endAddress:
					continue
				if type == self.TYPE_DATA:
					if len(bin) < addr - addrBias + count: # Reallocate
						bytesToAdd = addr - addrBias + count - len(bin)
						for i in range(bytesToAdd):
							defOffs = len(bin) % len(defaultBytes)
							bin += [ defaultBytes[defOffs], ]
					for i in range(9, 9 + count * 2, 2):
						byte = int2byte(int(line[i:i+2], 16))
						offset = (i - 9) // 2 + addr - addrBias
						if bin[offset] != defaultBytes[offset % len(defaultBytes)] and \
						   not doublewriteWarned:
							doublewriteWarned = True
							print "Invalid IHEX format (Wrote twice to same location)"
						bin[offset] = byte
					continue
				raise TOPException("Invalid IHEX format (unsup type %d)" % type)
		except ValueError:
			raise TOPException("Invalid IHEX format (digit format)")
		return b"".join(bin)

	def fromBinary(self, binData):
		ihex = []
		addr = 0
		for i in range(0, len(binData), 16):
			if addr > 0xFFFF:
				checksum = 0
				ihex.append(":%02X%04X%02X" % (2, 0, self.TYPE_ELAR))
				checksum += 2 + 0 + 0 + self.TYPE_ELAR
				a = (addr >> 16) & 0xFFFF
				ihex.append("%04X" % a)
				checksum += ((a >> 8) & 0xFF) + (a & 0xFF)
				checksum = ((checksum ^ 0xFF) + 1) & 0xFF
				ihex.append("%02X\n" % checksum)
				addr -= 0xFFFF
			checksum = 0
			size = min(len(binData) - i, 16)
			ihex.append(":%02X%04X%02X" % (size, addr, self.TYPE_DATA))
			checksum += size + ((addr >> 8) & 0xFF) + (addr & 0xFF) + self.TYPE_DATA
			for j in range(0, size):
				data = byte2int(binData[i + j])
				checksum = (checksum + data) & 0xFF
				ihex.append("%02X" % data)
			checksum = ((checksum ^ 0xFF) + 1) & 0xFF
			ihex.append("%02X\n" % checksum)
			addr += size
		ihex.append(":00000001FF\n")
		return "".join(ihex)

class IO_ahex(object):
	def autodetect(self, data):
		try:
			self.toBinary(data)
		except (TOPException), e:
			return False
		return True

	def toBinary(self, data, addressRange=None, defaultBytes=b"\xFF"):
		# defaultBytes is ignored
		binData = parseHexdump(data)
		if addressRange:
			binData = binData[addressRange.startAddress : addressRange.endAddress + 1]
		return binData

	def fromBinary(self, data):
		return generateHexdump(data)

class IO_binary(object):
	def autodetect(self, data):
		return True

	def toBinary(self, data, addressRange=None, defaultBytes=b"\xFF"):
		# defaultBytes is ignored
		if addressRange:
			data = data[addressRange.startAddress : addressRange.endAddress + 1]
		return data

	def fromBinary(self, data):
		return data

def IO_autodetect(data):
	"Returns an IO_... object for the data."
	if IO_ihex().autodetect(data):
		return IO_ihex
	elif IO_ahex().autodetect(data):
		return IO_ahex
	elif IO_binary().autodetect(data):
		return IO_binary
	assert(0) # Can't reach, because binary will always match.



# Import all chip modules in **ALPHABETICAL** order

from _27cxxx import *
from _74hc4094 import *
from at89c2051dip20 import *
from at89s51dip40 import *
from at89s52dip40 import *
from atmega168dip28 import *
from atmega32dip40 import *
from atmega328dip28 import *
from atmega48dip28 import *
from atmega88dip28 import *
from atmega8dip28 import *
from attiny13dip8 import *
from attiny26dip20 import *
from hm62256dip28 import *
from m24cxxdip8 import *
from m8cissp import *
from unitest import *
from w29ee011dip32 import *
from microchip8 import *
from microchip16 import *



"""
#    TOP2049 Open Source programming suite
#
#    IHEX file layout
#
#    Copyright (c) 2013 Michael Buesch <m@bues.ch>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

from util import *


class AddressRange(object):
	"""Address range type."""

	def __init__(self, startAddress = 0, endAddress = -1):
		# endAddress of -1 means "to the end of the image".
		assert(startAddress >= 0)
		assert(endAddress >= -1)
		assert(startAddress <= endAddress or endAddress == -1)
		self.startAddress = startAddress
		self.endAddress = endAddress

	def overlaps(self, other):
		"""Check if self overlaps with other."""
		if (other.startAddress <= self.endAddress or self.endAddress < 0) and\
		   (other.endAddress >= self.startAddress or other.endAddress < 0):
			return True
		return False

	@classmethod
	def overlapsAny(cls, addressRangeList, addressRange):
		"""Check if 'addressRange' overlaps with any
		AddressRange in 'addressRangeList'"""
		return any(r.overlaps(addressRange) for r in addressRangeList)

class IHexInterpreter(object):
	"""Generic interpreter for a cumulative IHEX file.
	This class must be subclassed to be usable."""

	# Definition of the address ranges.
	# For each memory area type, several possible address
	# ranges can be defined. They are tried first to last.
	# Override these in the subclass.
	progmemRanges = None
	eepromRanges = None
	fuseRanges = None
	lockRanges = None
	ramRanges = None
	uilRanges = None

	# Definition of default memory values.
	# Override these in the subclass.
	progmemDefaultBytes = b"\xFF"
	eepromDefaultBytes = b"\xFF"
	fuseDefaultBytes = b"\xFF"
	lockDefaultBytes = b"\xFF"
	ramDefaultBytes = b"\x00"
	uilDefaultBytes = b"\xFF"

	def __init__(self):
		self.__ihexData = None
		self.__progmem = None
		self.__eeprom = None
		self.__fusebits = None
		self.__lockbits = None
		self.__ram = None
		self.__uil = None

	def cumulativeSupported(self):
		"""Returns True, if parsing of cumulative IHEX files
		is supported by the implementation."""
		return self.progmemRanges or\
		       self.eepromRanges or\
		       self.fuseRanges or\
		       self.lockRanges or\
		       self.ramRanges or\
		       self.uilRanges

	def interpret(self, ihexData):
		self.__ihexData = ihexData
		usedRanges = []
		self.__progmem = self.tryExtract(ihexData, self.progmemDefaultBytes,
						 usedRanges, self.progmemRanges)
		self.__eeprom = self.tryExtract(ihexData, self.eepromDefaultBytes,
						usedRanges, self.eepromRanges)
		self.__fusebits = self.tryExtract(ihexData, self.fuseDefaultBytes,
						  usedRanges, self.fuseRanges)
		self.__lockbits = self.tryExtract(ihexData, self.lockDefaultBytes,
						  usedRanges, self.lockRanges)
		self.__ram = self.tryExtract(ihexData, self.ramDefaultBytes,
					     usedRanges, self.ramRanges)
		self.__uil = self.tryExtract(ihexData, self.uilDefaultBytes,
					     usedRanges, self.uilRanges)

	def tryExtract(self, ihexData, defaultBytes, alreadyUsedRanges, tryRanges):
		if not tryRanges:
			return None
		for tryRange in tryRanges:
			if AddressRange.overlapsAny(alreadyUsedRanges, tryRange):
				continue
			image = IO_ihex().toBinary(ihexData,
						   addressRange = tryRange,
						   defaultBytes = defaultBytes)
			if not image:
				continue
			alreadyUsedRanges.append(tryRange)
			return image
		return None

	def getRaw(self, defaultBytes=b"\xFF"):
		return IO_ihex().toBinary(self.__ihexData,
					  defaultBytes = defaultBytes)

	def getProgmem(self, dontInterpretSections=False):
		if dontInterpretSections:
			return self.getRaw(defaultBytes = self.progmemDefaultBytes)
		return self.__progmem

	def getEEPROM(self, dontInterpretSections=False):
		if dontInterpretSections:
			return self.getRaw(defaultBytes = self.eepromDefaultBytes)
		return self.__eeprom

	def getFusebits(self, dontInterpretSections=False):
		if dontInterpretSections:
			return self.getRaw(defaultBytes = self.fuseDefaultBytes)
		return self.__fusebits

	def getLockbits(self, dontInterpretSections=False):
		if dontInterpretSections:
			return self.getRaw(defaultBytes = self.lockDefaultBytes)
		return self.__lockbits

	def getRAM(self, dontInterpretSections=False):
		if dontInterpretSections:
			return self.getRaw(defaultBytes = self.ramDefaultBytes)
		return self.__ram

	def getUIL(self, dontInterpretSections=False):
		if dontInterpretSections:
			return self.getRaw(defaultBytes = self.uilDefaultBytes)
		return self.__uil
